import click
from loguru import logger

import komodo_cli.printing as printing
from komodo_cli.backends.backend import Backend
from komodo_cli.backends.backend_factory import BackendFactory
from komodo_cli.types import (JobNotFoundException,
                              JobNotInDesiredStateException, JobStatus)
from komodo_cli.utils import APIClient, update_context_with_api_client


@click.command("cancel")
@click.argument(
    "job_id",
    type=str,
)
@click.pass_context
def cmd_cancel(ctx: click.Context, job_id: str):
    """Cancel a Komodo job."""
    update_context_with_api_client(ctx)
    logger.info(f"Job ID: {job_id}")

    api_client: APIClient = ctx.obj["api_client"]
    job = api_client.get_job(job_id)

    backend_schema = api_client.get_backend(job.backend_name)
    backend: Backend = BackendFactory.get_backend(
        backend_schema,
        api_client,
    )

    printing.info(f"Cancelling job {job_id}", bold=True)
    try:
        backend.cancel(job.backend_job_id)
    except (JobNotFoundException, JobNotInDesiredStateException) as e:
        printing.error(str(e), bold=True)
        return
    except Exception as e:
        printing.error(f"Unexpected exception: {str(e)}", bold=True)
        return

    try:
        api_client.update_job(job_id, JobStatus.CANCELLED, job.backend_job_id)
    except Exception as e:
        printing.error(
            f"Error updating job {job_id} status to {JobStatus.CANCELLED.value}",
            bold=True,
        )
        raise e



import click
from tabulate import tabulate

import komodo_cli.printing as printing
from komodo_cli.utils import APIClient, update_context_with_api_client


@click.command("list")
@click.option("--skip", "-s", type=int, default=0)
@click.option("--limit", "-l", type=int, default=10)
@click.pass_context
def cmd_list(ctx: click.Context, skip: int, limit: int):
    """List Komodo jobs."""
    update_context_with_api_client(ctx)
    api_client: APIClient = ctx.obj["api_client"]

    try:
        jobs = api_client.list_jobs(skip, limit)

        jobs_to_print = [
            [
                job.id,
                job.command,
                job.status.value,
                job.backend_name,
            ]
            for job in jobs
        ]

        printing.header(f"Found {len(jobs)} Komodo jobs\n", bold=True)
        printing.info(
            tabulate(
                jobs_to_print,
                headers=[
                    "Job ID",
                    "Command",
                    "Status",
                    "Backend",
                ],
                tablefmt="simple_grid",
            ),
        )
    except Exception as e:
        printing.error(f"Error listing jobs", bold=True)
        raise e



import click
from loguru import logger

import komodo_cli.printing as printing
from komodo_cli.backends.backend import Backend
from komodo_cli.backends.backend_factory import BackendFactory
from komodo_cli.types import (JobNotFoundException,
                              JobNotInDesiredStateException, JobStatus,
                              LocalJobNotFoundException)
from komodo_cli.utils import APIClient, update_context_with_api_client


@click.command("delete")
@click.argument(
    "job_id",
    type=str,
)
@click.option(
    "--force",
    "-f",
    type=bool,
    is_flag=True,
    help="Set this flag if you want to delete the job even if the job is still running",
)
@click.pass_context
def cmd_delete(ctx: click.Context, job_id: str, force: bool):
    """Delete a Komodo job."""
    update_context_with_api_client(ctx)
    logger.info(f"Job ID: {job_id}")

    api_client: APIClient = ctx.obj["api_client"]

    if job_id == "all":
        jobs = api_client.list_jobs()
    else:
        try:
            jobs = [api_client.get_job(job_id)]
        except JobNotFoundException:
            printing.error(f"Job {job_id} does not exist", bold=True)
            return

    for job in jobs:
        backend_schema = api_client.get_backend(job.backend_name)
        backend: Backend = BackendFactory.get_backend(
            backend_schema,
            api_client,
        )

        printing.warning(f"Deleting job {job.id}", bold=True)
        try:
            if not force and job.status not in [
                JobStatus.CANCELLED,
                JobStatus.CANCELLING,
                JobStatus.ERROR,
                JobStatus.UNKNOWN,
                JobStatus.FINISHED,
                JobStatus.NOT_FOUND,
            ]:
                printing.error(
                    f"Job {job.id} has not finished, and --force flag was not set",
                    bold=True,
                )
                return
            backend.delete(job.backend_job_id)
        except JobNotFoundException as e:
            printing.error(f"Job {job.id} does not exist", bold=True)
            return
        except LocalJobNotFoundException:
            if not force:
                printing.info(
                    f"Local job cannot be found on this machine. Use --force to delete anyways."
                )
                return
        except Exception as e:
            printing.error(f"Unexpected exception: {str(e)}", bold=True)
            raise e

        try:
            api_client.delete_job(job.id, force)
        except (JobNotFoundException, JobNotInDesiredStateException) as e:
            if not force:
                printing.error(str(e), bold=True)
                raise e
        except Exception as e:
            printing.error(
                f"Error deleting job {job.id}",
                bold=True,
            )
            raise e



import click
from loguru import logger
from tabulate import tabulate

import komodo_cli.printing as printing
from komodo_cli.utils import APIClient, update_context_with_api_client


@click.command("status")
@click.argument(
    "job_id",
    type=str,
)
@click.pass_context
def cmd_status(ctx: click.Context, job_id: str):
    """Get status of a Komodo job."""
    update_context_with_api_client(ctx)
    logger.info(f"Job ID: {job_id}")

    api_client: APIClient = ctx.obj["api_client"]

    try:
        job = api_client.get_job(job_id)
        jobs_to_print = [
            [
                job.id,
                job.command,
                job.status.value,
                job.backend_name,
                job.resource_name,
                job.backend_job_id,
            ]
        ]

        click.echo(
            tabulate(
                jobs_to_print,
                headers=[
                    "Job ID",
                    "Command",
                    "Status",
                    "Backend",
                    "Resource",
                    "Backend Job ID",
                ],
                tablefmt="simple_grid",
            ),
        )
    except Exception as e:
        printing.error(f"Error getting job {job_id}", bold=True)
        raise e



import sys

import click
from loguru import logger

import komodo_cli.printing as printing
from komodo_cli.backends.backend import Backend
from komodo_cli.backends.backend_factory import BackendFactory
from komodo_cli.types import JobNotFoundException, JobStatus
from komodo_cli.utils import APIClient, update_context_with_api_client


@click.command("logs")
@click.argument(
    "job_id",
    type=str,
)
@click.option(
    "--node-index",
    "-i",
    type=int,
    default=0,
    help="The index of the node to get logs from",
)
@click.option(
    "--watch",
    "-w",
    type=bool,
    is_flag=True,
    help="Whether to watch the logs or just output current logs.",
)
@click.pass_context
def cmd_logs(ctx: click.Context, job_id: str, node_index: int, watch: bool):
    """Get logs for a Komodo job."""
    update_context_with_api_client(ctx)
    logger.info(f"Job ID: {job_id}")
    logger.info(f"Watch: {watch}")

    api_client: APIClient = ctx.obj["api_client"]

    try:
        job = api_client.get_job(job_id)
        if job.status == JobStatus.PENDING:
            printing.warning(
                f"Job {job_id} is currently pending",
                bold=True,
            )
            return
        replica_id = api_client.get_job_replica_id(job_id, node_index)
    except:
        printing.error(
            f"Error getting job {job_id}",
            bold=True,
        )
        sys.exit(1)

    backend_schema = api_client.get_backend(job.backend_name)
    backend: Backend = BackendFactory.get_backend(
        backend_schema,
        api_client,
    )

    printing.info(f"Getting logs for job {job_id}", bold=True)
    try:
        for line in backend.logs(replica_id, watch):
            if type(line) != str:
                line = line.decode("utf-8")
            printing.info(f"{line.strip()}")
    except JobNotFoundException as e:
        printing.error(str(e), bold=True)
    except Exception as e:
        printing.error(f"Unexpected exception: {str(e)}", bold=True)
